import docopt


class ValidationException(Exception):
    pass


def docopt_wrapper(
        doc,
        argv=None,
        help_=True,
        version=None,
        options_first=False,
        transforms=None,
        validations=None
    ):
    args = docopt.docopt(doc, argv, help_, version, options_first)


    for argument_key, validate in validations.items():
        try:
            arg = args[argument_key]
        except KeyError:
            raise ValidationException(
                '%s is not a valid argument for validation.' % argument_key
            )

        try:
            success = validate(arg)
            if not success:
                raise Exception
        except Exception:
            raise ValidationException(
                'Could not validate parameter %s.' % argument_key
            )

    for argument_key, transform in transforms.items():
        try:
            arg = args[argument_key]
        except KeyError:
            raise ValidationException(
                '%s is not a valid argument for transformation.' % argument_key
            )

        try:
            arg = transform(arg)
        except Exception:
            raise ValidationException(
                'Could not transform parameter %s.' % argument_key
            )

        args[argument_key] = arg


    return args
