import collections
import cPickle as pickle
import errno
import os


class Pickler(object):
    def __init__(self, default, filename):
        pickledir = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'pickled'
        )

        try:
            os.makedirs(pickledir)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise

        self.filepath = os.path.join(
            pickledir,
            filename
        )

        try:
            with open(self.filepath, 'rb') as f:
                self.store = pickle.load(f)
        except (EOFError, IOError):
            self.store = default


    def save(self):
        # TODO: incremental update
        with open(self.filepath, 'wb') as f:
            pickle.dump(self.store, f)


class DictPickler(Pickler, collections.MutableMapping):
    def __init__(self, filename):
        Pickler.__init__(self, dict(), filename)


    def __getitem__(self, key):
        return self.store[self.__keytransform__(key)]

    def __setitem__(self, key, value):
        self.store[self.__keytransform__(key)] = value

        self.save()

    def __delitem__(self, key):
        del self.store[self.__keytransform__(key)]

        self.save()

    def __iter__(self):
        return iter(self.store)

    def __len__(self):
        return len(self.store)

    # pylint: disable=R0201
    def __keytransform__(self, key):
        return key


    def __str__(self):
        return self.store.__str__()

    def __repr__(self):
        return self.store.__repr__()
